@@ -11,20 +11,38 @@ use serde::{Deserialize, Serialize};
 use std::collections::VecDeque;
 
 ///
-/// This type refer to `Transaction` structs where inputs are
+/// This type refer to `Block` structs where inputs are
 /// replaced by connected outputs.
 ///
 /// ## Implementors:
-/// - STransaction
-/// - FTransaction
+/// - SConnectedBlock
+/// - FConnectedBlock
 ///
-pub trait TxConnectable {
-    type TOut: 'static + From<TxOut> + Send;
+pub trait ConnectedBlock {
+    ///
+    /// Associated output type.
+    ///
+    type Tx: ConnectedTx + Send;
 
-    fn from(tx: &Transaction) -> Self;
-    fn add_input(&mut self, input: Self::TOut);
+    ///
+    /// Construct a ConnectedBlock from parts of a block.
+    ///
+    /// Used in `iter_connected.rs`.
+    ///
+    fn from(block_header: BlockHeader, block_hash: BlockHash) -> Self;
+
+    ///
+    /// Add a new transaction in this block.
+    ///
+    /// Used in `iter_connected.rs`.
+    ///
+    fn add_tx(&mut self, tx: Self::Tx);
+
+    ///
+    /// Construct a ConnectedBlock and connect the transactions.
+    ///
     fn connect(
-        tx: Transaction,
+        block: Block,
         tx_db: &TxDB,
         blk_index: &BlockIndex,
         blk_file: &BlkFile,
@@ -34,20 +52,39 @@ pub trait TxConnectable {
 }
 
 ///
-/// This type refer to `Block` structs where inputs are
+/// This type refer to `Transaction` structs where inputs are
 /// replaced by connected outputs.
 ///
 /// ## Implementors:
-/// - SConnectedBlock
-/// - FConnectedBlock
+/// - STransaction
+/// - FTransaction
 ///
-pub trait BlockConnectable {
-    type Tx: TxConnectable + Send;
+pub trait ConnectedTx {
+    ///
+    /// Associated output type.
+    ///
+    type TOut: 'static + From<TxOut> + Send;
 
-    fn from(block_header: BlockHeader, block_hash: BlockHash) -> Self;
-    fn add_tx(&mut self, tx: Self::Tx);
+    ///
+    /// Construct a ConnectedTx from Transaction without blank inputs.
+    ///
+    /// This function is used in `iter_connected.rs`.
+    ///
+    fn from(tx: &Transaction) -> Self;
+
+    ///
+    /// Add a input to this ConnectedTx.
+    ///
+    /// This function is used in `iter_connected.rs`.
+    ///
+    fn add_input(&mut self, input: Self::TOut);
+
+    ///
+    /// Build ConnectedTx from Tx,
+    /// and attach inputs to this ConnectedTx using tx-index.
+    ///
     fn connect(
-        block: Block,
+        tx: Transaction,
         tx_db: &TxDB,
         blk_index: &BlockIndex,
         blk_file: &BlkFile,
@@ -83,9 +120,7 @@ pub struct FConnectedBlock {
 #[derive(Serialize, Deserialize, Clone, PartialEq, Eq, Debug)]
 pub struct SConnectedTransaction {
     pub txid: Txid,
-    /// List of inputs
     pub input: Vec<STxOut>,
-    /// List of outputs
     pub output: Vec<STxOut>,
 }
 
@@ -98,13 +133,11 @@ pub struct FConnectedTransaction {
     pub version: i32,
     pub lock_time: u32,
     pub txid: Txid,
-    /// List of inputs
     pub input: Vec<FTxOut>,
-    /// List of outputs
     pub output: Vec<FTxOut>,
 }
 
-impl TxConnectable for FConnectedTransaction {
+impl ConnectedTx for FConnectedTransaction {
     type TOut = FTxOut;
 
     fn from(tx: &Transaction) -> Self {
@@ -132,7 +165,7 @@ impl TxConnectable for FConnectedTransaction {
             version: tx.version,
             lock_time: tx.lock_time,
             txid: tx.txid(),
-            input: connect_output_tx_in(tx.input, is_coinbase, tx_db, blk_index, blk_file)?
+            input: connect_tx_inputs(&tx.input, is_coinbase, tx_db, blk_index, blk_file)?
                 .into_iter()
                 .map(|x| x.into())
                 .collect(),
@@ -141,7 +174,7 @@ impl TxConnectable for FConnectedTransaction {
     }
 }
 
-impl TxConnectable for SConnectedTransaction {
+impl ConnectedTx for SConnectedTransaction {
     type TOut = STxOut;
 
     fn from(tx: &Transaction) -> Self {
@@ -165,7 +198,7 @@ impl TxConnectable for SConnectedTransaction {
         let is_coinbase = tx.is_coin_base();
         Ok(SConnectedTransaction {
             txid: tx.txid(),
-            input: connect_output_tx_in(tx.input, is_coinbase, tx_db, blk_index, blk_file)?
+            input: connect_tx_inputs(&tx.input, is_coinbase, tx_db, blk_index, blk_file)?
                 .into_iter()
                 .map(|x| x.into())
                 .collect(),
@@ -174,7 +207,7 @@ impl TxConnectable for SConnectedTransaction {
     }
 }
 
-impl BlockConnectable for FConnectedBlock {
+impl ConnectedBlock for FConnectedBlock {
     type Tx = FConnectedTransaction;
 
     fn from(block_header: BlockHeader, block_hash: BlockHash) -> Self {
@@ -197,12 +230,12 @@ impl BlockConnectable for FConnectedBlock {
         let block_hash = block.header.block_hash();
         Ok(FConnectedBlock {
             header: FBlockHeader::parse(block.header, block_hash),
-            txdata: connect_output(block.txdata, tx_db, blk_index, blk_file)?,
+            txdata: connect_block_inputs(block.txdata, tx_db, blk_index, blk_file)?,
         })
     }
 }
 
-impl BlockConnectable for SConnectedBlock {
+impl ConnectedBlock for SConnectedBlock {
     type Tx = SConnectedTransaction;
 
     fn from(block_header: BlockHeader, block_hash: BlockHash) -> Self {
@@ -225,17 +258,104 @@ impl BlockConnectable for SConnectedBlock {
         let block_hash = block.header.block_hash();
         Ok(SConnectedBlock {
             header: SBlockHeader::parse(block.header, block_hash),
-            txdata: connect_output(block.txdata, tx_db, blk_index, blk_file)?,
+            txdata: connect_block_inputs(block.txdata, tx_db, blk_index, blk_file)?,
         })
     }
 }
 
 ///
-/// This function converts outpoints to outputs. It converts:
+/// This function is used for connecting transaction inputs for a single block.
+///
+#[inline]
+fn connect_block_inputs<Tx>(
+    transactions: Vec<Transaction>,
+    tx_db: &TxDB,
+    blk_index: &BlockIndex,
+    blk_file: &BlkFile,
+) -> OpResult<Vec<Tx>>
+where
+    Tx: ConnectedTx,
+{
+    // collect all inputs
+    let mut all_tx_in = Vec::with_capacity(transactions.len());
+    for tx in &transactions {
+        for tx_in in &tx.input {
+            all_tx_in.push(tx_in);
+        }
+    }
+
+    // connect transactions inputs in parallel
+    let mut connected_outputs: VecDeque<Option<TxOut>> = all_tx_in
+        .par_iter()
+        .map(|x| connect_input(x, tx_db, blk_index, blk_file))
+        .collect();
+
+    // reconstruct block
+    let mut connected_tx = Vec::with_capacity(transactions.len());
+    for tx in transactions {
+        let outpoints_count = if tx.is_coin_base() { 0 } else { tx.input.len() };
+
+        let mut outputs = Vec::with_capacity(outpoints_count);
+        for _ in 0..tx.input.len() {
+            let connected_out = connected_outputs.pop_front().unwrap();
+            if let Some(out) = connected_out {
+                // also do not push the null input connected to coinbase transaction
+                outputs.push(out);
+            }
+        }
+        // check if any output is missing
+        if outputs.len() != outpoints_count {
+            return Err(OpError::from(
+                "some outpoints aren't found, tx_index is not fully synced",
+            ));
+        }
+        let mut tx = Tx::from(&tx);
+        for o in outputs {
+            tx.add_input(o.into());
+        }
+        connected_tx.push(tx);
+    }
+    Ok(connected_tx)
+}
+
+///
+/// This function converts multiple Inputs of a single transaction to Outputs in parallel.
+///
+#[inline]
+fn connect_tx_inputs(
+    tx_in: &[TxIn],
+    is_coinbase: bool,
+    tx_db: &TxDB,
+    blk_index: &BlockIndex,
+    blk_file: &BlkFile,
+) -> OpResult<Vec<TxOut>> {
+    let connected_outputs: Vec<TxOut> = tx_in
+        .par_iter()
+        .filter_map(|x| connect_input(x, tx_db, blk_index, blk_file))
+        .collect();
+
+    let outpoints_count = if is_coinbase { 0 } else { tx_in.len() };
+    let received = connected_outputs.len();
+
+    // some outpoints aren't found
+    if received != outpoints_count {
+        Err(OpError::from(
+            format!("some outpoints aren't found, tx_index is not fully synced, (expected: {}, read: {}, txid)", outpoints_count, received).as_str(),
+        ))
+    } else {
+        Ok(connected_outputs)
+    }
+}
+
+///
+/// This function connect a single TxIn to outputs. It converts:
 /// - read failure to `None`
 /// - coinbase transaction output to `None`
 ///
-fn outpoint_connect(
+/// It is used in `connect_output_tx_in` and `connect_output`.
+///
+#[inline]
+fn connect_input(
     tx_in: &TxIn,
     tx_db: &TxDB,
     blk_index: &BlockIndex,
@@ -244,10 +364,11 @@ fn outpoint_connect(
     let outpoint = tx_in.previous_output;
     let tx_id = &outpoint.txid;
     let n = outpoint.vout;
+    // skip coinbase transaction
     if !is_coin_base(tx_in) {
-        // special treatment of genesis tx
+        // special treatment of genesis tx, which cannot be found in tx-index.
         if tx_db.is_genesis_tx(tx_id) {
-            return match blk_index.records.get(0) {
+            return match blk_index.records.first() {
                 None => None,
                 Some(pos) => match blk_file.read_block(pos.n_file, pos.n_data_pos) {
                     Ok(mut blk) => {
@@ -283,93 +404,6 @@ fn outpoint_connect(
     }
 }
 
-///
-/// This function is used for connecting transaction outpoints
-///
-#[inline]
-fn connect_output_tx_in(
-    tx_in: Vec<TxIn>,
-    is_coinbase: bool,
-    tx_db: &TxDB,
-    blk_index: &BlockIndex,
-    blk_file: &BlkFile,
-) -> OpResult<Vec<TxOut>> {
-    let connected_outputs: Vec<TxOut> = tx_in
-        .par_iter()
-        .filter_map(|x| outpoint_connect(x, tx_db, blk_index, blk_file))
-        .collect();
-
-    let outpoints_count = if is_coinbase { 0 } else { tx_in.len() };
-    let received = connected_outputs.len();
-
-    // some outpoints aren't found
-    if received != outpoints_count {
-        Err(OpError::from(
-            format!("some outpoints aren't found, tx_index is not fully synced, (expected: {}, read: {}, txid)", outpoints_count, received).as_str(),
-        ))
-    } else {
-        Ok(connected_outputs)
-    }
-}
-
-///
-/// This function is used for connecting transactions for blocks
-///
-fn connect_output<Tx>(
-    transactions: Vec<Transaction>,
-    tx_db: &TxDB,
-    blk_index: &BlockIndex,
-    blk_file: &BlkFile,
-) -> OpResult<Vec<Tx>>
-where
-    Tx: TxConnectable,
-{
-    let all_tx_in = get_all_tx_in(&transactions);
-
-    let mut connected_outputs: VecDeque<Option<TxOut>> = all_tx_in
-        .par_iter()
-        .map(|x| outpoint_connect(x, tx_db, blk_index, blk_file))
-        .collect();
-
-    // reconstruct block
-    let mut connected_tx = Vec::with_capacity(transactions.len());
-    for tx in transactions {
-        let outpoints_count = if tx.is_coin_base() { 0 } else { tx.input.len() };
-
-        let mut outputs = Vec::with_capacity(outpoints_count);
-        for _ in 0..tx.input.len() {
-            let connected_out = connected_outputs.pop_front().unwrap();
-            if let Some(out) = connected_out {
-                // also do not push the null input connected to coinbase transaction
-                outputs.push(out);
-            }
-        }
-        // check if any output is missing
-        if outputs.len() != outpoints_count {
-            return Err(OpError::from(
-                "some outpoints aren't found, tx_index is not fully synced",
-            ));
-        }
-        let mut tx = Tx::from(&tx);
-        for o in outputs {
-            tx.add_input(o.into());
-        }
-        connected_tx.push(tx);
-    }
-    Ok(connected_tx)
-}
-
-#[inline]
-fn get_all_tx_in(transactions: &Vec<Transaction>) -> Vec<&TxIn> {
-    let mut all_tx_in = Vec::with_capacity(transactions.len());
-    for tx in transactions {
-        for tx_in in &tx.input {
-            all_tx_in.push(tx_in);
-        }
-    }
-    all_tx_in
-}
-
 #[inline]
 fn is_coin_base(tx_in: &TxIn) -> bool {
     tx_in.previous_output.is_null()
